/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "BlockedThread.h"

#include "ThreadPool.h"
#include "OSManager.h"
#include "GenericEvent.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

CBlockedThread::CBlockedThread(CThreadPool::COMInit aInit) : m_pEvent(NULL)
{
	//Create the thread pool
	m_pPool=new CThreadPool(1,
							1,
							NULL,
							CGenericThread::tpNormal,
							aInit);

	//Create the event
	m_pEvent=COSManager::CreateEvent();

}

CBlockedThread::~CBlockedThread()
{
	//Get rid of the things
	delete m_pPool;
	delete m_pEvent;
}

DWORD CBlockedThread::BlockedCall(CThreadPool::LPThreadDWORDPoolProc pProc,
								  LPVOID lpData)
{
	//Build our data
	ThreadData aData;
	aData.lpData=lpData;
	aData.pClass=this;
	aData.pEvent=m_pEvent;
	aData.pProc=pProc;
	aData.dwReturnValue=0;

	//Call the thread
	m_pPool->SubmitJob(ThreadProc,
					   (LPVOID)&aData);

	//Now wait
	m_pEvent->Wait();

	//Done
	return aData.dwReturnValue;
}

void CBlockedThread::ThreadProc(LPVOID lpData)
{
	//Take our data
	ThreadData* pData;
	pData=(ThreadData*)lpData;

	//Call the function
	pData->dwReturnValue=(*pData->pProc)(pData->lpData);

	//Alarm the event
	pData->pEvent->Set();
}

KOMODIA_NAMESPACE_END